import pandas_datareader as pdr
import requests
import pandas as pd
from bs4 import BeautifulSoup

# 네이버 차트에서 수정주가(종가, 시가)
def get_data_naver(company_code):
    # count=3000에서 3000은 과거 3,000 영업일간의 데이터를 의미. 사용자가 조절 가능
    url = "https://fchart.stock.naver.com/sise.nhn?symbol={}&timeframe=day&count=300000&requestType=0".format(company_code)
    get_result = requests.get(url)
    bs_obj = BeautifulSoup(get_result.content, "html.parser")

    # information
    inf = bs_obj.select('item')
    columns = ['date', 'Open', 'High', 'Low', 'Close', 'Volume']
    df_inf = pd.DataFrame([], columns=columns, index=range(len(inf)))

    for i in range(len(inf)):
        df_inf.iloc[i] = str(inf[i]['data']).split('|')
    df_inf.index = pd.to_datetime(df_inf['date'])
    return df_inf.drop('date', axis=1).astype(float)
def get_naver_open_close(company_codes):
    output = pd.DataFrame()
    if type(company_codes)==str:
        company_code=company_codes

        df = get_data_naver(company_code)[['Open', 'Close']].stack().reset_index().rename(columns={0:company_code})
        df['level_1'] = df['level_1'].apply(lambda x:"-16" if x=='Close' else "-09")
        df.index = pd.to_datetime(df['date'].astype(str) + df['level_1'])
        output = df[[company_code]]
    else:
        for company_code in company_codes:
            df = get_data_naver(company_code)[['Open', 'Close']].stack().reset_index().rename(columns={0: company_code})
            df['level_1'] = df['level_1'].apply(lambda x: "-16" if x == 'Close' else "-09")
            df.index = pd.to_datetime(df['date'].astype(str) + df['level_1'])
            output = pd.concat([output, df[[company_code]]], axis=1)
    return output
def get_naver_close(company_codes):
    output = pd.DataFrame()
    if type(company_codes)==str:
        company_code=company_codes
        df = get_data_naver(company_code)[['Close']].rename(columns={'Close':company_code})
        output = df[[company_code]]
    else:
        for company_code in company_codes:
            company_code = company_codes
            df = get_data_naver(company_code)[['Close']].rename(columns={'Close': company_code})
            output = pd.concat([output, df[[company_code]]], axis=1)
    return output


# 야후 수정주가 가져오기
def get_all_yahoo_data(name):
    return pdr.get_data_yahoo(name, start='1980-01-01').rename_axis('date', axis=0).sort_index()
def get_data_yahoo_close(symbols):
    if type(symbols) ==str:
        df = get_all_yahoo_data(symbols)[['Adj Close']].rename(columns={'Adj Close':symbols})
    else:
        df = get_all_yahoo_data(symbols)['Adj Close']
    return df
def get_data_yahoo_open_close(name):
    df = get_all_yahoo_data(name)

    close = df['Adj Close']
    open = df['Adj Close'] * df['Open'] / df['Close']
    open.index = pd.to_datetime(open.index.astype(str) + '-09')
    close.index = pd.to_datetime(close.index.astype(str) + '-16')
    ans = pd.concat([open,close]).sort_index()
    if type(name) == str:
        ans.name = name
        ans = pd.DataFrame(ans)
    else:
        ans.index.name = 'date'
        ans.columns.name = None
    return ans


import pandas as pd
import GD_utils as gdu
class calculator:
    def __init__(self, ratio_df, cost=0.0073):
        """
        :param ratio_df:    (
                            index   - rebalancing dates
                            columns - Symbols(the same with imported price data)
                            value   - weight
                            )

        :param cost:        (
                            [%]
                            ex. if do you want to apply trading cost of 0.73%, cost=0.0074
                            )
        """
        self.rb_dt = ratio_df.copy()
        self.rb_dts = ratio_df.index
        self.p_dt = gdu.data.copy().loc[self.rb_dts[0]:,ratio_df.columns]
        self.cost = cost

        gr_p = self.get_df_grouped(self.p_dt, self.rb_dts)

        # 회전율 관련
        self.daily_ratio = gr_p.apply(self.calc_bt_daily_ratio)
        self.rebal_date_turnover_ratio = self.calc_rb_turnover(self.daily_ratio)

        # 수익률 관련
        # periodic cumulative return
        # self.cmpd_rtn_period = gr_p.apply(self.calc_compound_return)

        # back-test daily return
        self.backtest_daily_return = gr_p.apply(self.calc_bt_compound_return).droplevel(0)
        # back-test daily cumulative return
        self.backtest_cumulative_return = self.backtest_daily_return.add(1).cumprod()


    def get_df_grouped(self, df, dts):
        # df, dts = self.p_dt.copy(), self.rb_dts.copy()
        df.loc[dts,'gr_idx'] = dts
        df['gr_idx'] = df['gr_idx'].fillna(method='ffill')
        return df.groupby('gr_idx')

    def calc_compound_return(self, grouped_price):
        return grouped_price.drop('gr_idx', axis=1).pct_change().fillna(0).add(1).cumprod().sub(1)
    def calc_bt_compound_return(self, grouped_price):
        # grouped_price = gr_p.get_group([x for x in gr_p.groups.keys()][0])
        gr_rtn = grouped_price.set_index('gr_idx').pct_change().fillna(0).add(1).cumprod().sub(1)
        output = gr_rtn.mul(self.rb_dt.loc[gr_rtn.index[0]]).add(1).pct_change().sum(1).fillna(0)
        output.index = grouped_price.index

        # apply trading cost
        rb_d = output.index[0]
        output.loc[rb_d] = output.loc[rb_d] - self.cost*self.rb_tr_ratio.loc[rb_d]
        return output
    def calc_bt_daily_ratio(self, grouped_price):
        gr_rtn = grouped_price.set_index('gr_idx').pct_change().fillna(0).add(1).cumprod()#.sub(1)
        output = gr_rtn.mul(self.rb_dt.loc[gr_rtn.index[0]])#.add(1).pct_change().sum(1).fillna(0)
        output.index = grouped_price.index
        return output
    def calc_rb_turnover(self, daily_account_ratio):
        rb_ratio_diff = daily_account_ratio.diff(1).loc[self.rb_dts]
        rb_ratio_diff.loc[self.rb_dts[0], self.rb_dt.columns] = self.rb_dt.iloc[0]
        return abs(rb_ratio_diff).sum(1)


import setuptools

setuptools.setup(
    name="GD_utils",
    version="0.0.1",
    license='MIT',
    author="GD park",
    author_email="gdpresent@naver.com",
    description="backtest tools",
    long_description=open('README.md').read(),
    url="https://github.com/gdpresent/GD_utils.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)